#include <ros/ros.h>
#include <scamper_move/ScamperMoveAction.h>
#include <actionlib/client/simple_action_client.h>

using namespace scamper_move;
using actionlib::SimpleActionClient;
using actionlib::SimpleClientGoalState;
typedef SimpleActionClient<ScamperMoveAction> Client;

/*** Action完了時に呼び出されるコールバック関数 ***/
void onActionDone(const SimpleClientGoalState &state,
                  const ScamperMoveResultConstPtr &result)
{
  ROS_INFO("Result message : %s", result->msg.c_str());
}
/*** Action起動時に呼び出されるコールバック関数 ***/
void onActionActivate()
{
  ROS_INFO("Action was activated");
}
/*** Feedback受信時に呼び出されるコールバック関数 ***/
void onFeedbackReceived(const ScamperMoveFeedbackConstPtr &feedback)
{
  ROS_INFO("Pose : (%.2f, %.2f)", 
    feedback->pose.position.x, feedback->pose.position.y);
}

int main(int argc, char** argv)
{
  /*** ROSノードの初期化 ***/
  ros::init(argc, argv, "move_test");
  /*** actionlib clientの初期化 ***/
  Client client("omni_move", true);
  client.waitForServer();
  /*** 目標地点のリストを生成 ***/
  std::array<std::array<double, 2>, 8> target_pts = 
  {
    std::array<double, 2>{0.5, .0}, std::array<double, 2>{-0.5, .0},
    std::array<double, 2>{.0, 0.5}, std::array<double, 2>{.0, -0.5},
    std::array<double, 2>{0.354, 0.354}, std::array<double, 2>{-0.354, -0.354},
    std::array<double, 2>{0.354, -0.354}, std::array<double, 2>{-0.354, 0.354}
  };
  /*** Scamperを縦横斜め8方向に移動 ***/
  ScamperMoveGoal goal;
  for(int i = 0; i < target_pts.size(); i++){
    /*** 目標地点の設定 ***/
    goal.target_pt.x = target_pts[i][0];
    goal.target_pt.y = target_pts[i][1];
    /*** Actionの呼び出し ***/
    client.sendGoal(goal, &onActionDone, &onActionActivate, &onFeedbackReceived);
    /*** Actionが終了するまで待機 ***/
    while(ros::ok() && client.getState() != SimpleClientGoalState::SUCCEEDED){
      ros::Duration(0.1).sleep();
    }
    ros::Duration(1.0).sleep();
  }
  return 0;
}