#include <ros/ros.h>
#include <tutorial_service/CountNumberAction.h>
#include <actionlib/client/simple_action_client.h>

using namespace tutorial_service;
using actionlib::SimpleActionClient;
using actionlib::SimpleClientGoalState;
typedef SimpleActionClient<CountNumberAction> Client;

/*** Action完了時に呼び出されるコールバック関数 ***/
void onActionDone(const SimpleClientGoalState &state,
                  const CountNumberResultConstPtr &result)
{
  ROS_INFO("Result message : %s", result->msg.c_str());
}
/*** Action起動時に呼び出されるコールバック関数 ***/
void onActionActivate()
{
  ROS_INFO("Action was activated");
}
/*** Feedback受信時に呼び出されるコールバック関数 ***/
void onFeedbackReceived(const CountNumberFeedbackConstPtr &feedback)
{
  ROS_INFO("Feedback : %d", feedback->current_val);
}

int main(int argc, char** argv)
{
  /*** ROSノードの初期化 ***/
  ros::init(argc, argv, "action_client");
  /*** actionlib clientの初期化 ***/
  Client client("count_number", true);
  /*** Serverが初期化されるまで待機 ***/
  client.waitForServer();
  /*** goalの設定 ***/
  CountNumberGoal goal;
  goal.max_val = 10;
  /*** Actionの呼び出し ***/
  client.sendGoal(goal, &onActionDone, &onActionActivate, &onFeedbackReceived);
  /*** Actionが終了するまで待機 ***/
  while(ros::ok() && client.getState() != SimpleClientGoalState::SUCCEEDED){
    ROS_INFO("Processing action...");
    ros::Duration(1.0).sleep();
  }
  ROS_INFO("Action was done");
  return 0;
}