#include <ros/ros.h>
#include <tutorial_service/CountNumberAction.h>
#include <actionlib/server/simple_action_server.h>

using namespace tutorial_service;
using actionlib::SimpleActionServer;
typedef SimpleActionServer<CountNumberAction> Server;

/*** Actionのコールバック関数 ***/
void actCountNumber(const CountNumberGoalConstPtr &goal, Server* server)
{
  CountNumberFeedback feedback;
  CountNumberResult result;
  ros::Rate loop_rate(1);
  /*** 1秒ごとにカウントをインクリメント ***/
  /*** カウントがmax_valに達したら終了***/
  /*** インクリメントと同時にfeedbackを送信 ***/
  for(int i = 0; i < goal->max_val; i++){
    ROS_INFO("Count : %d", i);
    feedback.current_val = i;
    server->publishFeedback(feedback);
    loop_rate.sleep();
  }
  /*** resultにメッセージを代入 ***/
  result.msg = "Succeeded";
  /*** Actionの完了処理 ***/
  server->setSucceeded(result);
}

int main(int argc, char** argv)
{
  /*** ROSの初期化 ***/
  ros::init(argc, argv, "action_server");
  ros::NodeHandle nh;
  /*** Action serverの初期化 ***/
  Server server(nh, "count_number", boost::bind(&actCountNumber, _1, &server), false);
  /*** Action serverの開始 ***/
  server.start();
  /*** コールバックを待機 ***/
  ros::spin();
  return 0;
}