#include <ros/ros.h>
#include <std_msgs/String.h>
#include <std_srvs/Empty.h>

using std_msgs::String;
using std_srvs::EmptyRequest;
using std_srvs::EmptyResponse;

/*** Topicを受信すると呼ばれるコールバック関数 ***/
void onTextSubscribed(const String &str)
{
  ROS_INFO("I heard %s", str.data.c_str());
}

/*** Serviceのコールバック関数 ***/
bool serviceIsAlive(EmptyRequest &req, EmptyResponse &res)
{
  return true;
}

int main(int argc, char **argv)
{
  /*** ROSノードの初期化 ***/
  ros::init(argc, argv, "remote_server");
  ros::NodeHandle nh;
  /*** Parameterの設定 ***/
  nh.setParam("text", "Remote test");
  /*** SubscribeするTopicの設定＆コールバック関数の登録 ***/
  ros::Subscriber sub = nh.subscribe("text", 10, onTextSubscribed);
  /*** 提供するServiceの登録 ***/
  ros::ServiceServer srv = nh.advertiseService("is_alive", serviceIsAlive);
  /*** コールバックを待機 ***/
  ros::spin();
  return 0;
}